package com.kindlebook.hyrt.chapter12;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Hashtable;

import org.xmlpull.v1.XmlPullParserException;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.res.XmlResourceParser;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageSwitcher;
import android.widget.ImageView;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.ViewSwitcher;

public class QuizGameActivity extends QuizActivity {
    private static final String DEBUG_TAG = "QuizGameActivity";
    private TextSwitcher mBookTitleSwitcher;
    private ImageSwitcher mBookCoverSwitcher;
    private SharedPreferences mGameSettings;
    private Hashtable<Integer, Book> mBooks;

    private static final boolean mDebug = true;
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.game);

        // Set up Text Switcher
        mBookTitleSwitcher = (TextSwitcher) findViewById(R.id.bookTitleTextSwitcher);
        mBookTitleSwitcher.setFactory(new BookTitleSwitcherFactory());

        // Set up Image Switcher
        mBookCoverSwitcher = (ImageSwitcher) findViewById(R.id.bookCoverImageSwitcher);
        mBookCoverSwitcher.setFactory(new BookCoverSwitcherFactory());

        // Retrieve the shared preferences
        mGameSettings = getSharedPreferences(GAME_PREFERENCES,
                Context.MODE_PRIVATE);

        // Initialize books batch
        mBooks = new Hashtable<Integer, Book>(BOOK_BATCH_SIZE);

        // get current book
        int startingBookNumber = mGameSettings.getInt(
                GAME_PREFERENCES_CURRENT_BOOK, 0);

        // If we're at the beginning of the quiz, initialize the Shared
        // preferences
        if (startingBookNumber == 0 || mDebug == true) {
            Editor editor = mGameSettings.edit();
            editor.putInt(GAME_PREFERENCES_CURRENT_BOOK, 1);
            editor.commit();
            startingBookNumber = 1;
        }

        try {
            loadBookBatch(startingBookNumber);
        } catch (Exception e) {
            Log.e(DEBUG_TAG, "Loading initial book batch failed", e);
        }

        // If the book was loaded properly, display it
        if (mBooks.containsKey(startingBookNumber) == true) {
            // Set the text of the textswitcher
            mBookTitleSwitcher.setCurrentText(getBookTitle(startingBookNumber));

            // Set the image of the imageswitcher
            Drawable image = getBookCoverImageDrawable(startingBookNumber);
            mBookCoverSwitcher.setImageDrawable(image);
        } else {
            // Tell the user we don't have any new books at this time
            handleNoBooks();
        }

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.gameoptions, menu);
        menu.findItem(R.id.help_options_menu_item).setIntent(
                new Intent(this, QuizHelpActivity.class));
        menu.findItem(R.id.settings_options_menu_item).setIntent(
                new Intent(this, QuizSettingsActivity.class));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        super.onOptionsItemSelected(item);
        startActivity(item.getIntent());
        return true;
    }

    public void onNoButton(View v) {
        handleAnswerAndShowNextBook(false);
    }

    public void onYesButton(View v) {
        handleAnswerAndShowNextBook(true);
    }

    /**
     * 
     * Helper method to record the answer the user gave and load up the next
     * book.
     * 
     * @param bAnswer
     *            The answer the user gave
     */
    private void handleAnswerAndShowNextBook(boolean bAnswer) {
        int curScore = mGameSettings.getInt(GAME_PREFERENCES_SCORE, 0);
        int nextBookNumber = mGameSettings.getInt(
                GAME_PREFERENCES_CURRENT_BOOK, 1) + 1;

        Editor editor = mGameSettings.edit();
        editor.putInt(GAME_PREFERENCES_CURRENT_BOOK, nextBookNumber);

        // Log the number of "yes" answers only
        if (bAnswer == true) {
            editor.putInt(GAME_PREFERENCES_SCORE, curScore + 1);
        }
        editor.commit();

        if (mBooks.containsKey(nextBookNumber) == false) {
            // Load next batch before checking again
            try {
                loadBookBatch(nextBookNumber);
            } catch (Exception e) {
                Log.e(DEBUG_TAG, "Loading updated book batch failed", e);
            }
        }

        if (mBooks.containsKey(nextBookNumber) == true) {
            // Update book text
            mBookTitleSwitcher.setText(getBookTitle(nextBookNumber));

            // Update book image
            Drawable image = getBookCoverImageDrawable(nextBookNumber);
            mBookCoverSwitcher.setImageDrawable(image);
        } else {
            handleNoBooks();
        }

    }

    /**
     * Helper method to configure the quiz screen when no books were found.
     * Could be called for a variety of error cases, including no new books, IO
     * failures, or parser failures.
     */
    private void handleNoBooks() {
        mBookTitleSwitcher.setText(getResources().getText(
                R.string.quiz_no_books));
        mBookCoverSwitcher.setImageResource(R.drawable.nobook);

        // Disable yes button
        Button yesButton = (Button) findViewById(R.id.yesButton);
        yesButton.setEnabled(false);

        // Disable no button
        Button noButton = (Button) findViewById(R.id.noButton);
        noButton.setEnabled(false);
    }

    /**
     * Returns a {@code String} representing the text for a particular book
     * number
     * 
     * @param bookNumber
     *            The book number to get the title for
     * @return The book title, or null if {@code bookNumber} not found
     */
    private String getBookTitle(Integer bookNumber) {
        String text = null;
        Book book = (Book) mBooks.get(bookNumber);
        if (book != null) {
            text = book.mTitle;
        }
        return text;
    }

    /**
     * Retrieves a {@code Drawable} object (cover image) for a particular book
     * 
     * @param bookNumber
     *            The book number to get the {@code Drawable} for
     * @return A {@code Drawable} for the particular book, or a placeholder
     *         image if the loading failed or the book doesn't exist
     */
    private Drawable getBookCoverImageDrawable(int bookNumber) {
        Drawable image;
        URL imageUrl;
        try {
            imageUrl = new URL(getBookCoverUrl(bookNumber));
            InputStream stream = imageUrl.openStream();
            Bitmap bitmap = BitmapFactory.decodeStream(stream);
            image = new BitmapDrawable(getResources(), bitmap);
        } catch (Exception e) {
            Log.e(DEBUG_TAG, "Decoding Bitmap stream failed", e);
            image = getResources().getDrawable(R.drawable.nobook);
        }
        return image;
    }

    /**
     * Returns a {@code String} representing the URL to a book cover
     * 
     * @param bookNumber
     *            The book number to get the URL for
     * @return A {@code String} for the URL or null if none found
     */
    private String getBookCoverUrl(Integer bookNumber) {
        String url = null;
        Book book = (Book) mBooks.get(bookNumber);
        if (book != null) {
            url = book.mImageUrl;
        }
        return url;
    }

    /**
     * Loads the XML into the {@see mBooks} class member variable
     * 
     * @param startBookNumber
     *            retrieve from this number and higher
     * @throws XmlPullParserException
     *             Thrown if XML parsing errors
     * @throws IOException
     *             Thrown if errors loading XML
     */
    private void loadBookBatch(int startBookNumber)
            throws XmlPullParserException, IOException {
        // Remove old batch
        mBooks.clear();

        // TODO: Contact the server and retrieve a batch of book data, beginning
        // at startBookNumber
        XmlResourceParser bookBatch;

        // TEST: BEGIN MOCK BOOKS
        if (startBookNumber < 16) {
            bookBatch = getResources().getXml(R.xml.mock_booklist_1);
        } else {
            bookBatch = getResources().getXml(R.xml.mock_booklist_2);
        }
        // TEST: END MOCK BOOKS

        // Parse the XML
        int eventType = -1;

        // Find Score records from XML
        while (eventType != XmlResourceParser.END_DOCUMENT) {
            if (eventType == XmlResourceParser.START_TAG) {

                // Get the name of the tag (eg books or book)
                String strName = bookBatch.getName();

                if (strName.equals(XML_TAG_BOOK)) {

                    String bookNumber = bookBatch.getAttributeValue(null,
                            XML_TAG_BOOK_ATTRIBUTE_NUMBER);
                    Integer bookNum = new Integer(bookNumber);
                    String bookTitle = bookBatch.getAttributeValue(null,
                            XML_TAG_BOOK_ATTRIBUTE_TITLE);
                    String bookCoverUrl = bookBatch.getAttributeValue(null,
                            XML_TAG_BOOK_ATTRIBUTE_COVERURL);

                    // Save data to our hashtable
                    mBooks.put(bookNum, new Book(bookNum, bookTitle,
                            bookCoverUrl));
                }
            }
            eventType = bookBatch.next();
        }
    }

    /**
     * A switcher factory for use with the book cover image. Creates the next
     * {@code ImageView} object to animate to
     * 
     */
    private class BookCoverSwitcherFactory implements ViewSwitcher.ViewFactory {
        public View makeView() {
            ImageView imageView = (ImageView) LayoutInflater.from(
                    getApplicationContext()).inflate(
                    R.layout.book_cover_switcher_view, mBookCoverSwitcher,
                    false);
            return imageView;
        }
    }

    /**
     * A switcher factory for use with the book title text. Creates the next
     * {@code TextView} object to animate to
     * 
     */
    private class BookTitleSwitcherFactory implements ViewSwitcher.ViewFactory {
        public View makeView() {
            TextView textView = (TextView) LayoutInflater.from(
                    getApplicationContext()).inflate(
                    R.layout.book_title_switcher_view, mBookTitleSwitcher,
                    false);
            return textView;
        }
    }

    /**
     * Object to manage the data for a single book
     * 
     */
    private class Book {
        @SuppressWarnings("unused")
        int mNumber;
        @SuppressWarnings("unused")
        String mTitle;
        @SuppressWarnings("unused")
        String mImageUrl;

        /**
         * 
         * Constructs a new Book object
         * 
         * @param bookNum
         *            The number of this book
         * @param bookTitle
         *            The title for this book
         * @param bookCoverUrl
         *            A valid image Url to display with this book
         */
        @SuppressWarnings("unused")
        public Book(int bookNum, String bookTitle, String bookCoverUrl) {
            mNumber = bookNum;
            mTitle = bookTitle;
            mImageUrl = bookCoverUrl;
        }
    }

}